list_json = [
    {
        "number": "43-0000",
        "name": "Precinct 01 Turf 01",
        "eventSignups": None,
        "listSize": 62,
    },
    {
        "number": "44-0000",
        "name": "Precinct 01 Turf 02",
        "eventSignups": None,
        "listSize": 54,
    },
    {
        "number": "45-0000",
        "name": "Precinct 01 Turf 03",
        "eventSignups": None,
        "listSize": 70,
    },
    {
        "number": "46-0000",
        "name": "Precinct 01 Turf 04",
        "eventSignups": None,
        "listSize": 66,
    },
    {
        "number": "48-0000",
        "name": "Precinct 02 Turf 01",
        "eventSignups": None,
        "listSize": 45,
    },
    {
        "number": "49-0000",
        "name": "Precinct 02 Turf 02",
        "eventSignups": None,
        "listSize": 33,
    },
    {
        "number": "50-0000",
        "name": "Precinct 02 Turf 03",
        "eventSignups": None,
        "listSize": 63,
    },
    {
        "number": "51-0000",
        "name": "Precinct 02 Turf 04",
        "eventSignups": None,
        "listSize": 66,
    },
    {
        "number": "52-0000",
        "name": "Precinct 02 Turf 05",
        "eventSignups": None,
        "listSize": 39,
    },
    {
        "number": "55-0000",
        "name": "Precinct 03 Turf 01",
        "eventSignups": None,
        "listSize": 53,
    },
    {
        "number": "56-0000",
        "name": "Precinct 03 Turf 02",
        "eventSignups": None,
        "listSize": 45,
    },
    {
        "number": "57-0000",
        "name": "Precinct 03 Turf 03",
        "eventSignups": None,
        "listSize": 50,
    },
    {
        "number": "58-0000",
        "name": "Precinct 03 Turf 04",
        "eventSignups": None,
        "listSize": 54,
    },
    {
        "number": "59-0000",
        "name": "Precinct 03 Turf 05",
        "eventSignups": None,
        "listSize": 65,
    },
]

single_list_json = {
    "number": "43-0000",
    "name": "Precinct 01 Turf 01",
    "eventSignups": [],
    "listSize": 62,
}
