GET_PEOPLE_RESPONSE = {
    "results": [
        {
            "birthdate": None,
            "city_district": None,
            "civicrm_id": None,
            "county_district": None,
            "county_file_id": None,
            "created_at": "2023-06-22T11:41:56-07:00",
            "datatrust_id": None,
            "do_not_call": False,
            "do_not_contact": False,
            "dw_id": None,
            "email": "foo@example.com",
            "email_opt_in": True,
            "employer": None,
            "external_id": None,
            "federal_district": None,
            "fire_district": None,
            "first_name": "Foo",
            "has_facebook": False,
            "id": 4,
            "is_twitter_follower": False,
            "is_volunteer": False,
            "judicial_district": None,
            "labour_region": None,
            "last_name": "Bar",
            "linkedin_id": None,
            "mobile": None,
            "mobile_opt_in": None,
            "middle_name": "",
            "nbec_guid": None,
            "ngp_id": None,
            "note": None,
            "occupation": None,
            "party": None,
            "pf_strat_id": None,
            "phone": None,
            "precinct_id": None,
            "primary_address": None,
            "profile_image_url_ssl": "https://example.com/assets/notifier/profile-avatar.png",
            "recruiter_id": None,
            "rnc_id": None,
            "rnc_regid": None,
            "salesforce_id": None,
            "school_district": None,
            "school_sub_district": None,
            "sex": None,
            "signup_type": 0,
            "state_file_id": None,
            "state_lower_district": None,
            "state_upper_district": None,
            "support_level": None,
            "supranational_district": None,
            "tags": ["zoot", "boot"],
            "twitter_id": None,
            "twitter_name": None,
            "updated_at": "2023-06-22T11:41:56-07:00",
            "van_id": None,
            "village_district": None,
            "ward": None,
            "work_phone_number": None,
        },
        {
            "birthdate": None,
            "city_district": None,
            "civicrm_id": None,
            "county_district": None,
            "county_file_id": None,
            "created_at": "2023-06-22T08:21:00-07:00",
            "datatrust_id": None,
            "do_not_call": False,
            "do_not_contact": False,
            "dw_id": None,
            "email": "bar@example.com",
            "email_opt_in": True,
            "employer": None,
            "external_id": None,
            "federal_district": None,
            "fire_district": None,
            "first_name": "Zoo",
            "has_facebook": False,
            "id": 2,
            "is_twitter_follower": False,
            "is_volunteer": False,
            "judicial_district": None,
            "labour_region": None,
            "last_name": "Baz",
            "linkedin_id": None,
            "mobile": None,
            "mobile_opt_in": True,
            "middle_name": "",
            "nbec_guid": None,
            "ngp_id": None,
            "note": None,
            "occupation": None,
            "party": None,
            "pf_strat_id": None,
            "phone": None,
            "precinct_id": None,
            "primary_address": None,
            "profile_image_url_ssl": "https://example.com/assets/notifier/profile-avatar.png",
            "recruiter_id": None,
            "rnc_id": None,
            "rnc_regid": None,
            "salesforce_id": None,
            "school_district": None,
            "school_sub_district": None,
            "sex": None,
            "signup_type": 0,
            "state_file_id": None,
            "state_lower_district": None,
            "state_upper_district": None,
            "support_level": None,
            "supranational_district": None,
            "tags": ["zoo", "bar"],
            "twitter_id": None,
            "twitter_name": None,
            "updated_at": "2023-06-22T11:43:03-07:00",
            "van_id": None,
            "village_district": None,
            "ward": None,
            "work_phone_number": None,
        },
    ],
    "next": None,
    "prev": None,
}

PERSON_RESPONSE = {
    "person": {
        "birthdate": None,
        "city_district": None,
        "civicrm_id": None,
        "county_district": None,
        "county_file_id": None,
        "created_at": "2023-06-22T08:21:00-07:00",
        "datatrust_id": None,
        "do_not_call": False,
        "do_not_contact": False,
        "dw_id": None,
        "email": "foo@example.com",
        "email_opt_in": True,
        "employer": None,
        "external_id": None,
        "federal_district": None,
        "fire_district": None,
        "first_name": "Foo",
        "has_facebook": False,
        "id": 1,
        "is_twitter_follower": False,
        "is_volunteer": False,
        "judicial_district": None,
        "labour_region": None,
        "last_name": "Bar",
        "linkedin_id": None,
        "mobile": None,
        "mobile_opt_in": True,
        "middle_name": "",
        "nbec_guid": None,
        "ngp_id": None,
        "note": None,
        "occupation": None,
        "party": None,
        "pf_strat_id": None,
        "phone": None,
        "precinct_id": None,
        "primary_address": None,
        "profile_image_url_ssl": "https://example.com/assets/notifier/profile-avatar.png",
        "recruiter_id": None,
        "rnc_id": None,
        "rnc_regid": None,
        "salesforce_id": None,
        "school_district": None,
        "school_sub_district": None,
        "sex": None,
        "signup_type": 0,
        "state_file_id": None,
        "state_lower_district": None,
        "state_upper_district": None,
        "support_level": None,
        "supranational_district": None,
        "tags": [],
        "twitter_id": None,
        "twitter_name": None,
        "updated_at": "2023-06-22T08:21:00-07:00",
        "van_id": None,
        "village_district": None,
        "ward": None,
        "work_phone_number": None,
        "active_customer_expires_at": None,
        "active_customer_started_at": None,
        "author": None,
        "author_id": None,
        "auto_import_id": None,
        "availability": None,
        "ballots": [],
        "banned_at": None,
        "billing_address": None,
        "bio": None,
        "call_status_id": None,
        "call_status_name": None,
        "capital_amount_in_cents": 500,
        "children_count": 0,
        "church": None,
        "city_sub_district": None,
        "closed_invoices_amount_in_cents": None,
        "closed_invoices_count": None,
        "contact_status_id": None,
        "contact_status_name": None,
        "could_vote_status": False,
        "demo": None,
        "donations_amount_in_cents": 0,
        "donations_amount_this_cycle_in_cents": 0,
        "donations_count": 0,
        "donations_count_this_cycle": 0,
        "donations_pledged_amount_in_cents": 0,
        "donations_raised_amount_in_cents": 0,
        "donations_raised_amount_this_cycle_in_cents": 0,
        "donations_raised_count": 0,
        "donations_raised_count_this_cycle": 0,
        "donations_to_raise_amount_in_cents": 0,
        "email1": "foo@example.com",
        "email1_is_bad": False,
        "email2": None,
        "email2_is_bad": False,
        "email3": None,
        "email3_is_bad": False,
        "email4": None,
        "email4_is_bad": False,
        "emails": [
            {
                "email_address": "foo@example.com",
                "email_number": 1,
                "is_bad": False,
                "is_primary": True,
            }
        ],
        "ethnicity": None,
        "facebook_address": None,
        "facebook_profile_url": None,
        "facebook_updated_at": None,
        "facebook_username": None,
        "fax_number": None,
        "federal_donotcall": False,
        "first_donated_at": None,
        "first_fundraised_at": None,
        "first_invoice_at": None,
        "first_prospect_at": None,
        "first_recruited_at": None,
        "first_supporter_at": "2023-06-22T08:21:00-07:00",
        "first_volunteer_at": None,
        "full_name": "Foo Bar",
        "home_address": None,
        "import_id": None,
        "inferred_party": None,
        "inferred_support_level": None,
        "invoice_payments_amount_in_cents": None,
        "invoice_payments_referred_amount_in_cents": None,
        "invoices_amount_in_cents": None,
        "invoices_count": None,
        "is_absentee_voter": None,
        "is_active_voter": None,
        "is_deceased": False,
        "is_donor": False,
        "is_dropped_from_file": None,
        "is_early_voter": None,
        "is_fundraiser": False,
        "is_ignore_donation_limits": False,
        "is_leaderboardable": True,
        "is_mobile_bad": False,
        "is_permanent_absentee_voter": None,
        "is_possible_duplicate": False,
        "is_profile_private": False,
        "is_profile_searchable": True,
        "is_prospect": False,
        "is_supporter": True,
        "is_survey_question_private": False,
        "language": None,
        "last_call_id": None,
        "last_contacted_at": None,
        "last_contacted_by": None,
        "last_donated_at": None,
        "last_fundraised_at": None,
        "last_invoice_at": None,
        "last_rule_violation_at": None,
        "legal_name": None,
        "locale": None,
        "mailing_address": None,
        "marital_status": None,
        "media_market_name": None,
        "meetup_id": None,
        "meetup_address": None,
        "mobile_normalized": None,
        "nbec_precinct_code": None,
        "nbec_precinct": None,
        "note_updated_at": None,
        "outstanding_invoices_amount_in_cents": None,
        "outstanding_invoices_count": None,
        "overdue_invoices_count": None,
        "page_slug": None,
        "parent": None,
        "parent_id": None,
        "party_member": None,
        "phone_normalized": None,
        "phone_time": None,
        "precinct_code": None,
        "precinct_name": None,
        "prefix": None,
        "previous_party": None,
        "primary_email_id": 1,
        "priority_level": None,
        "priority_level_changed_at": None,
        "profile_content": None,
        "profile_content_html": None,
        "profile_headline": None,
        "received_capital_amount_in_cents": 500,
        "recruiter": None,
        "recruits_count": 0,
        "registered_address": None,
        "registered_at": None,
        "religion": None,
        "rule_violations_count": 0,
        "signup_sources": [],
        "spent_capital_amount_in_cents": 0,
        "submitted_address": None,
        "subnations": [],
        "suffix": None,
        "support_level_changed_at": None,
        "support_probability_score": None,
        "township": None,
        "turnout_probability_score": None,
        "twitter_address": None,
        "twitter_description": None,
        "twitter_followers_count": None,
        "twitter_friends_count": None,
        "twitter_location": None,
        "twitter_login": None,
        "twitter_updated_at": None,
        "twitter_website": None,
        "unsubscribed_at": None,
        "user_submitted_address": None,
        "username": None,
        "voter_updated_at": None,
        "warnings_count": 0,
        "website": None,
        "work_address": None,
    },
    "precinct": None,
}
